#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2014
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : aix_track.tcl
##
## Purpose    : Mambo AIX process tracking
##
## CVS INFO:
##
## $Id$
##
#####################################################
#####################################################

###################################################
# Package initialization
###################################################

namespace eval AIXTrack {
    variable version            "1.0"
    variable hooks

    array set hooks [list\
    	0x10000000 KERN_FLIH\
    	0x10100000 KERN_SVC\
    	0x10200000 KERN_SLIH\
    	0x10300000 KERN_SLIHRET\
    	0x10400000 KERN_SYSCRET\
    	0x10500000 KERN_LVM\
    	0x10600000 KERN_DISPATCH\
    	0x10700000 VNOP_LOOKUP\
    	0x10A00000 KERN_PFS\
    	0x10B00000 KERN_LVMSIMP\
    	0x10C00000 KERN_IDLE\
    	0x10D00000 KERN_VFSINO\
    	0x10E00000 KERN_RELOCK\
    	0x10F00000 KERN_EOF\
    	0x11000000 KERN_STDERR\
    	0x11100000 KERN_LOCKF\
    	0x11200000 KERN_LOCK\
    	0x11300000 KERN_UNLOCK\
    	0x11400000 KERN_LOCKALLOC\
    	0x11500000 KERN_SETRECURSIVE\
    	0x11600000 KERN_XMALLOC\
    	0x11700000 KERN_XMFREE\
    	0x11800000 KERN_FORKCOPY\
    	0x11900000 KERN_PIDSIG\
    	0x11A00000 KERN_RCVSIGNAL\
    	0x11C00000 KERN_P_SLIH\
    	0x11D00000 KERN_SIG_SLIH\
    	0x11E00000 KERN_ISSIG\
    	0x11F00000 KERN_SORQ\
    	0x12000000 VNOP_ACCESS\
    	0x12100000 SYSC_ACCT\
    	0x12200000 SYSC_ALARM\
    	0x12300000 SYSC_AUDIT\
    	0x12400000 SYSC_AUDITLOG\
    	0x12500000 SYSC_AUDITPROC\
    	0x12600000 SYSC_AUDITSYS\
    	0x12800000 SYSC_CHMAC\
    	0x12900000 VNOP_SETATTR\
    	0x12C00000 SYSC_CHPRIV\
    	0x12E00000 VNOP_CLOSE\
    	0x13000000 VNOP_CREATE\
    	0x13100000 SYSC_DISCLAIM\
    	0x13400000 SYSC_EXECVE\
    	0x13500000 SYSC__EXIT\
    	0x13600000 VNOP_FCLEAR\
    	0x13900000 SYSC_FORK\
    	0x13A00000 VNOP_GETATTR\
    	0x13B00000 VFS_STATFS\
    	0x13C00000 VNOP_FSYNC\
    	0x13D00000 VNOP_FTRUNC\
    	0x13F00000 SYSC_GETEGID\
    	0x14000000 SYSC_GETEPRIV\
    	0x14100000 SYSC_GETEUID\
    	0x14200000 SYSC_GETGID\
    	0x14300000 SYSC_GETGROUPS\
    	0x14400000 SYSC_GETMPRIV\
    	0x14500000 SYSC_GETPGRP\
    	0x14600000 SYSC_GETPID\
    	0x14700000 SYSC_GETPPID\
    	0x14800000 SYSC_GETPRIV\
    	0x14900000 SYSC_GETTSTATE\
    	0x14A00000 SYSC_GETUID\
    	0x14B00000 SYSC_GTTY\
    	0x14C00000 VNOP_IOCTL\
    	0x14E00000 SYSC_KILL\
    	0x14F00000 SYSC_KPRINTF\
    	0x15000000 VNOP_LINK\
    	0x15100000 SYSC_LOADTBL\
    	0x15200000 VNOP_LOCKCTL\
    	0x15300000 SYSC_LOCKX\
    	0x15400000 VNOP_SEEK\
    	0x15500000 SYSC_MACCTL\
    	0x15600000 VNOP_MKDIR\
    	0x15900000 VFS_MOUNT\
    	0x15A00000 SYSC_NICE\
    	0x15B00000 VNOP_OPEN\
    	0x15D00000 SYSC_OUNAME\
    	0x15E00000 SYSC_PAUSE\
    	0x16000000 SYSC_PLOCK\
    	0x16100000 SYSC_PROFIL\
    	0x16200000 SYSC_PTRACE\
    	0x16300000 VNOP_RDWR_READ\
    	0x16400000 VNOP_READLINK\
    	0x16600000 SYSC_REBOOT\
    	0x16700000 VNOP_RENAME\
    	0x16800000 VNOP_RMDIR\
    	0x16900000 SYSC_SBREAK\
    	0x16A00000 SYSC_SELECT\
    	0x16B00000 SYSC_SETGID\
    	0x16C00000 SYSC_SETGROUPS\
    	0x16D00000 SYSC_SETMAC\
    	0x16E00000 SYSC_SETPGRP\
    	0x16F00000 SYSC_SETPRIO\
    	0x17000000 SYSC_SETPRIV\
    	0x17100000 SYSC_SETREUID\
    	0x17200000 SYSC_SETTSTATE\
    	0x17300000 SYSC_SETUID\
    	0x17B00000 DD_PCMKE\
    	0x18000000 SYSC_SIGACTION\
    	0x18100000 SYSC_SIGCLEANUP\
    	0x18200000 SYSC_SIGPROCMASK\
    	0x18300000 SYSC_SIGRETURN\
    	0x18400000 SYSC_SIGSTACK\
    	0x18500000 SYSC_SIGSUSPEND\
    	0x18900000 SYSC_STIME\
    	0x18A00000 SYSC_STTY\
    	0x18B00000 VNOP_SYMLINK\
    	0x18D00000 SYSC_TIME\
    	0x18E00000 SYSC_TIMES\
    	0x18F00000 SYSC_ULIMIT\
    	0x19000000 SYSC_UMASK\
    	0x19100000 VFS_UNMOUNT\
    	0x19400000 VNOP_REMOVE\
    	0x19500000 SYSC_USRINFO\
    	0x19700000 SYSC_UTIME\
    	0x19800000 SYSC_UTSSYS\
    	0x19B00000 SYSC_WAIT\
    	0x19C00000 VNOP_RDWR_WRITE\
    	0x1A000000 VNOP_READDIR\
    	0x1A200000 SYSC_UTIMES\
    	0x1A300000 SYSC_SLOOKUP\
    	0x1A400000 SYSC_GETRLIMIT\
    	0x1A500000 SYSC_SETRLIMIT\
    	0x1A600000 SYSC_GETRUSAGE\
    	0x1A700000 SYSC_GETPRIORITY\
    	0x1A800000 SYSC_SETPRIORITY\
    	0x1A900000 SYSC_ABSINTERVAL\
    	0x1AA00000 SYSC_GETINTERVAL\
    	0x1AB00000 SYSC_GETTIMER\
    	0x1AC00000 SYSC_INCINTERVAL\
    	0x1AD00000 SYSC_RESTIMER\
    	0x1AE00000 SYSC_RESABS\
    	0x1AF00000 SYSC_RESINC\
    	0x1B000000 VMM\
    	0x1B000000 VMM_ASSIGN\
    	0x1B100000 VMM_DELETE\
    	0x1B200000 VMM_PGEXCT\
    	0x1B300000 VMM_PROTEXCT\
    	0x1B400000 VMM_LOCKEXCT\
    	0x1B500000 VMM_RECLAIM\
    	0x1B600000 VMM_GETPARENT\
    	0x1B700000 VMM_COPYPARENT\
    	0x1B700001 VMM_MAPPARENT\
    	0x1B700002 VMM_COWRELOAD\
    	0x1B800000 VMM_VMAP\
    	0x1B900000 VMM_ZFOD\
    	0x1BA00000 VMM_PAGEOUT\
    	0x1BA00000 VMM_SIO\
    	0x1BA00001 VMM_PAGEIN\
    	0x1BA00002 VMM_SIO_PPIN\
    	0x1BA00003 VMM_SIO_PPOUT\
    	0x1BB00000 VMM_SEGCREATE\
    	0x1BC00000 VMM_SEGDELETE\
    	0x1BD00000 VMM_DALLOC\
    	0x1BD00001 VMM_DALLOC_PP\
    	0x1BD00002 VMM_DFREE\
    	0x1BD00003 VMM_DFREE_PPNFR\
    	0x1BD00004 VMM_DFREE_PPXPT\
    	0x1BE00000 VMM_PFENDOUT\
    	0x1BE00001 VMM_PFENDIN\
    	0x1BE00002 VMM_PFENDOUT_PP\
    	0x1BE00003 VMM_PFENDIN_PP\
    	0x1BF00000 VMM_EXCEPT\
    	0x1C000000 DD_STRATEGY\
    	0x1C100000 DD_START\
    	0x1C200000 DD_INTR\
    	0x1C300000 DD_GETBLK\
    	0x1C400000 DD_IOWAIT\
    	0x1C500000 DD_IODONE\
    	0x1C600000 DD_PHYSIO\
    	0x1C700000 DD_ATTACH\
    	0x1C800000 DD_PPDD\
    	0x1C900000 DD_CDDD\
    	0x1CA00000 DD_TAPEDD\
    	0x1CB00000 DD_CIODD\
    	0x1CC00000 DD_ETHDD\
    	0x1CD00000 DD_ENTDD\
    	0x1CE00000 DD_TOKDD\
    	0x1CF00000 DD_C327DD\
    	0x1D000000 DDSPEC_GRAPHIO\
    	0x1D100000 RAS_ERRLG\
    	0x1D200000 RAS_DUMP\
    	0x1D300000 LPP_PCSIM\
    	0x1D400000 WHIP_API\
    	0x1D500000 WHIP_DAEMON\
    	0x1D600000 WHIP_DRIVER\
    	0x1D700000 WHIP_EMUL\
    	0x1D800000 WHIP_FILE\
    	0x1D900000 VMM_ZERO_0\
    	0x1D900001 VMM_ZERO_1\
    	0x1DA00000 VMM_MKP\
    	0x1DB00000 VMM_FPGIN\
    	0x1DC00000 VMM_SPACEOK_0\
    	0x1DC00001 VMM_SPACEOK_1\
    	0x1DD00000 VMM_LRU\
    	0x1E000000 COMM\
    	0x1E000000 COMM_EM78\
    	0x1F000000 SYSC_SETTIMER\
    	0x1F100000 SYSC_GETARGS\
    	0x1F200000 SYSC_GETPROC\
    	0x1F300000 SYSC_GETUSER\
    	0x1F400000 SYSC_KNLIST\
    	0x1F500000 SYSC_SETSID\
    	0x1F600000 SYSC_SETPGID\
    	0x1F700000 SYSC_SETREGID\
    	0x1F800000 SYSC_SETPRI\
    	0x20000000 KERN_RESUME\
    	0x20100000 KERN_HFT\
    	0x20200000 KERN_KTSM\
    	0x20300000 KERN_CON\
    	0x20600000 KERN_SCHED_POST\
    	0x20700000 KERN_SCHED\
    	0x20800000 KERN_SCHED_STAT\
    	0x20900000 KERN_SCHED_STAT1\
    	0x20A00000 KERN_SCRUB_DISABLE\
    	0x20B00000 KERN_SCRUB_ENABLE\
    	0x20C00000 KERN_SCRUB_SEG\
    	0x20D00000 KERN_SCRUB_SOFT\
    	0x20E00000 KERN_LOCKL\
    	0x20F00000 KERN_UNLOCKL\
    	0x21000000 KERN_INITP\
    	0x22000000 DD_FDDD\
    	0x22100000 DD_SCDISKDD\
    	0x22200000 DD_BADISKDD\
    	0x22300000 DD_SCSIDD\
    	0x22400000 DD_MPQPDD\
    	0x22500000 DD_X25DD\
    	0x22600000 DD_GIO\
    	0x22700000 DD_X25PRF\
    	0x22800000 DD_SERDASDD\
    	0x22900000 DD_TMSCSIDD\
    	0x23000000 PCMCIA_SS\
    	0x23400000 KERN_PROF\
    	0x23500000 COB\
    	0x23600000 COB_RTE\
    	0x23700000 COB_APPL\
    	0x24000000 SYSX_DLC_START\
    	0x24100000 SYSX_DLC_HALT\
    	0x24200000 SYSX_DLC_TIMER\
    	0x24300000 SYSX_DLC_XMIT\
    	0x24400000 SYSX_DLC_RECV\
    	0x24500000 SYSX_DLC_PERF\
    	0x24600000 SYSX_DLC_MONITOR\
    	0x24800000 DD_SOL\
    	0x25000000 NETPERF\
    	0x25100000 NETERR\
    	0x25200000 SOCKET\
    	0x25400000 MBUF\
    	0x25500000 IFEN\
    	0x25600000 IFTR\
    	0x25700000 IFET\
    	0x25800000 IFXT\
    	0x25900000 IFSL\
    	0x25A00000 TCPDBG\
    	0x26000000 IFCA\
    	0x26100000 IFSO\
    	0x26200000 IFFD\
    	0x26300000 DD_FDDIDD\
    	0x27000000 SNA_COMM\
    	0x27100000 SNA_API\
    	0x27200000 PSLA_OPEN\
    	0x27300000 PSLA_CLOSE\
    	0x27400000 PSLA_READ\
    	0x27500000 PSLA_WRITE\
    	0x27600000 PSLA_IOCTL\
    	0x27700000 PSLA_INTR\
    	0x27800000 PSLA_CONF\
    	0x27900000 NCS_NCK\
    	0x28000000 HIA\
    	0x28100000 GSNA_API\
    	0x28200000 SNALIB_API\
    	0x28300000 NB_TRACE_NCB\
    	0x28400000 NB_TRACE_IOB\
    	0x28500000 NB_TRACE_OPEN\
    	0x28600000 NB_TRACE_COMM\
    	0x28700000 NB_TRACE_WRITE\
    	0x28800000 NB_TRACE_READ\
    	0x28900000 NB_TRACE_DATA\
    	0x28A00000 NB_TRACE_MIF\
    	0x28B00000 NB_TRACE_VALUE\
    	0x29000000 NETVIEW_6000\
    	0x29100000 HSTC_PRD\
    	0x29200000 VCA_DD\
    	0x2ED00000 NBC\
    	0x2FC00000 VMM_VWAIT\
    	0x2FD00000 RPDP\
    	0x2FE00000 SYSFR\
    	0x2FF00000 KERN_TPROF\
    	0x30000000 ODM_LIB\
    	0x34000000 NET_MALLOC\
    	0x34300000 PCMCIA_CS\
    	0x35000000 DD_CATDD\
    	0x38D00000 AIO\
    	0x38E00000 DD_SISADD\
    	0x38F00000 DR\
    	0x39000000 DD_SISRAID\
    	0x39300000 LVM_NONIO\
    	0x3A500000 ATMSOCK\
    	0x3B100000 DIO\
    	0x3B200000 DIO_IO\
    	0x3B300000 DIO_PSEG\
    	0x3B700000 SEC\
    	0x3B800000 SEC_DATA\
    	0x3C500000 SYSC_IPCACCESS\
    	0x3C600000 SYSC_IPCGET\
    	0x3C700000 SYSC_MSGCONV\
    	0x3C800000 SYSC_MSGCTL\
    	0x3C900000 SYSC_MSGGET\
    	0x3CA00000 SYSC_MSGRCV\
    	0x3CB00000 SYSC_MSGSELECT\
    	0x3CC00000 SYSC_MSGSND\
    	0x3CD00000 SYSC_MSGXRCV\
    	0x3CE00000 SYSC_SEMCONV\
    	0x3CF00000 SYSC_SEMCTL\
    	0x3D000000 SYSC_SEMGET\
    	0x3D100000 SYSC_SEMOP\
    	0x3D200000 SYSC_SEM_OP\
    	0x3D300000 SYSC_SHMAT\
    	0x3D400000 SYSC_SHMCONV\
    	0x3D500000 SYSC_SHMCTL\
    	0x3D600000 SYSC_SHMDT\
    	0x3D700000 SYSC_SHMGET\
    	0x3D800000 SYSC_MADVISE\
    	0x3D900000 SYSC_MINCORE\
    	0x3DA00000 SYSC_MMAP\
    	0x3DB00000 SYSC_MPROTECT\
    	0x3DC00000 SYSC_MSYNC\
    	0x3DD00000 SYSC_MUNMAP\
    	0x3DE00000 SYSC_MVALID\
    	0x3DF00000 SYSC_MSEM_INIT\
    	0x3E000000 SYSC_MSEM_LOCK\
    	0x3E100000 SYSC_MSEM_REMOVE\
    	0x3E200000 SYSC_MSEM_UNLOCK\
    	0x3EC00000 VFS_VGET\
    	0x40000000 STTY\
    	0x40100000 STTY_STRTTY\
    	0x40200000 STTY_LDTERM\
    	0x40300000 STTY_SPTR\
    	0x40400000 STTY_NLS\
    	0x40500000 STTY_PTY\
    	0x40600000 STTY_RS\
    	0x40700000 STTY_LION\
    	0x40800000 STTY_CXMA\
    	0x40900000 STTY_ISA\
    	0x40A00000 STTY_JAS\
    	0x41700000 VIRT_CONS\
    	0x41900000 CPU_PREEMPT\
    	0x46000000 KERN_ASSERTWAIT\
    	0x46100000 KERN_CLEARWAIT\
    	0x46200000 KERN_THREADBLOCK\
    	0x46300000 KERN_EMPSLEEP\
    	0x46400000 KERN_EWAKEUPONE\
    	0x46500000 SYSC_CRTHREAD\
    	0x46600000 KERN_KTHREADSTART\
    	0x46700000 SYSC_TERMTHREAD\
    	0x46800000 KERN_KSUSPEND\
    	0x46900000 SYSC_THREADSETSTATE\
    	0x46A00000 SYSC_THREADTERM_ACK\
    	0x46B00000 SYSC_THREADSETSCHED\
    	0x46C00000 KERN_TIDSIG\
    	0x46D00000 KERN_WAITLOCK\
    	0x46E00000 KERN_WAKEUPLOCK\
    	0x47600000 SYSC_THREADLOCKBOOST\
    	0x47700000 KERN_WAITLOCKLOCAL\
    	0x47800000 ACLE\
    	0x47B00000 SYSC_MLOCK\
    	0x47C00000 USBCDROM\
    	0x47F00000 KERN_PHANTOM_EXTINT\
    	0x48000000 VCM_MAIN\
    	0x48100000 VCM_DEBUG\
    	0x48E00000 PTHREAD_SPINLOCK\
    	0x48F00000 SEA\
    	0x49100000 KERN_IOPATH_DATA\
    	0x49200000 KERN_HCALL\
    	0x49600000 PTHREAD_BARRIER\
    	0x49A00000 AACCT\
    	0x49B00000 AACCT_DBG\
    	0x49C00000 KERN_EEH\
    	0x49D00000 KERN_DMA\
    	0x49E00000 CR\
    	0x4A500000 VMM_LRU2\
    	0x4A700000 DLPI\
    	0x4A800000 XTISO\
    	0x4A900000 TIMOD\
    	0x4AA00000 TIRDWR\
    	0x4AE00000 ERRHK\
    	0x4AF00000 KERN_DISABLEMENT\
    	0x4B000000 KERN_UNDISP\
    	0x4BD00000 TSIB\
    	0x4BE00000 TSIB_GENERAL\
    	0x4BF00000 TSIB_ERROR\
    	0x4C000000 TSIB_DEBUG\
    	0x4C100000 VMM_PSIZE_INC\
    	0x4C100001 VMM_PSIZE_DEC\
    	0x4C200000 VMM_PSM_PROM\
    	0x4C200001 VMM_PSM_PROM_SCAN\
    	0x4C200002 VMM_PSM_PCHANGE\
    	0x4C200003 VMM_PSM_LWPROM\
    	0x4C200004 VMM_PSM_LWMIG\
    	0x4C200005 VMM_PSM_HWPROM\
    	0x4C200006 VMM_PSM_DEM\
    	0x4C200007 VMM_PSM_DEM_SCAN\
    	0x4C200008 VMM_PSM_REQ\
    	0x4C300000 VMM_WRITE_0\
    	0x4C300001 VMM_WRITE_1\
    	0x4CA00000 VMM_ERR_REL_ALIGN\
    	0x4CA00001 VMM_ERR_PROT_ALIGN\
    	0x4CB00000 VNOP_FID\
    	0x4CC00000 VNOP_GETACL\
    	0x4CD00000 VNOP_SETACL\
    	0x4CE00000 VNOP_GETEA\
    	0x4CF00000 VNOP_SETEA\
    	0x4D000000 VNOP_LISTEA\
    	0x4D100000 VNOP_REMOVEEA\
    	0x4D200000 VNOP_GETPCL\
    	0x4D300000 VNOP_SETPCL\
    	0x4D400000 VNOP_REVOKE\
    	0x4D500000 VNOP_MEMCNTL\
    	0x4D600000 VFS_ROOT\
    	0x4D700000 VFS_SYNC\
    	0x4D800000 VFS_CNTL\
    	0x4D900000 VFS_QUOTACTL\
    	0x4DA00000 VFS_ACLXCNTL\
    	0x4DB00000 VNOP_HOLD\
    	0x4DC00000 PILE\
    	0x4DD00000 NLC\
    	0x4DE00000 J2_BUFMGR\
    	0x4DF00000 J2_ICACHE\
    	0x4E000000 J2_SNAPSHOT\
    	0x4E100000 J2_DMAP\
    	0x4E200000 J2_GLOBAL\
    	0x4E300000 J2_LOGMGR\
    	0x4E400000 J2_TXNMGR\
    	0x4E500000 KDM\
    	0x4EF00000 VNOP_MAP\
    	0x4F000000 VNOP_UNMAP\
    	0x4F100000 VNOP_FINFO\
    	0x4F200000 VNOP_STATEA\
    	0x4F900000 VMM_RESERVATION\
    	0x50000000 DISPLAY_VTSS_P\
    	0x50100000 DISPLAY_VTSS_D\
    	0x50200000 DISPLAY_RCM_P\
    	0x50300000 DISPLAY_RCM_D\
    	0x50400000 DISPLAY_DDF_P\
    	0x50500000 DISPLAY_DDF_D\
    	0x50600000 DISPLAY_GAIRMS_P\
    	0x50700000 DISPLAY_GAIRMS_D\
    	0x50800000 DISPLAY_GAI2DM1_P\
    	0x50900000 DISPLAY_GAI2DM1_D\
    	0x50A00000 DISPLAY_GAI3DM1_P\
    	0x50B00000 DISPLAY_GAI3DM1_D\
    	0x50C00000 DISPLAY_GAI3DM2_P\
    	0x50D00000 DISPLAY_GAI3DM2_D\
    	0x50E00000 DISPLAY_X11_P\
    	0x50F00000 DISPLAY_X11_D\
    	0x51000000 DISPLAY_X11EXT_P\
    	0x51100000 DISPLAY_X11EXT_D\
    	0x51200000 DISPLAY_GL_P\
    	0x51300000 DISPLAY_GL_D\
    	0x51400000 DISPLAY_GRAPHIGS_P\
    	0x51500000 DISPLAY_GRAPHIGS_D\
    	0x51600000 ACPA_P\
    	0x51700000 ACPA_D\
    	0x51800000 DISPLAY_RESERVED_518\
    	0x51900000 DISPLAY_RESERVED_519\
    	0x51A00000 DISPLAY_RESERVED_51A\
    	0x51B00000 DISPLAY_RESERVED_51B\
    	0x51C00000 DISPLAY_RESERVED_51C\
    	0x51D00000 DISPLAY_RESERVED_51D\
    	0x51E00000 DISPLAY_RESERVED_51E\
    	0x51F00000 DISPLAY_RESERVED_51F\
    	0x52600000 POLLCACHE\
    	0x52F00000 SEC_CRED\
    	0x53500000 TCP\
    	0x53600000 UDP\
    	0x53700000 IP\
    	0x53800000 IP6\
    	0x53900000 PCB\
    	0x59B00000 J2_IO\
    	0x5A000000 LDR\
    	0x5A100000 LDR_KMOD\
    	0x5A200000 LDR_PROC\
    	0x5A300000 LDR_ERR\
    	0x5A400000 LDR_CHKPT\
    	0x5A500000 LDR_NAMEDSHLIB\
    	0x5AA00000 SYSC_RTCLOCK\
    	0x5AB00000 SYSC_RTTIMER\
    	0x5AC00000 SYSC_PRIO_REQUEUE\
    	0x5AD00000 SYSC_PRIO_SCHED\
    	0x5B500000 SYSC_RTSEM\
    	0x5B600000 SYSC_RTMQ\
    	0x5B700000 SYSC_RTSHM\
    	0x5B800000 RTIPC_NAME\
    	0x5B900000 NETWLM\
    	0x5BA00000 DSM_MAIN\
    	0x5BC00000 RTAS_CALLS\
    	0x60000000 PTHREAD_SCHEDULER\
    	0x60300000 PTHREAD_TIMER\
    	0x60500000 PTHREAD_VPSLEEP\
    	0x60600000 PTHREAD_COND\
    	0x60700000 PTHREAD_MUTEX\
    	0x60800000 PTHREAD_RWLOCK\
    	0x60900000 PTHREAD_GENERAL\
    	0x60A00000 LIBC_MALL_SUBSYSTEM\
    	0x60B00000 LIBC_MALL_INTERNAL\
    	0x60C00000 SYSC_UKEY_PROTECT\
    	0x60D00000 SYSC_UKEY_GETKEY\
    	0x60E00000 LIBC_SPAWN_FATHER\
    	0x60F00000 LIBC_SPAWN_CHILD\
    	0x61500000 NET\
    	0x61700000 ROUTE\
    	0x61800000 LOOP\
    	0x61900000 CDLI\
    	0x61A00000 NETINET_MISC\
    	0x61B00000 SOCKET_UNIX\
    	0x62400000 VMSVC_PIN\
    	0x62400002 VMSVC_UNPIN\
    	0x62400004 VMSVC_XLATE_PIN\
    	0x62400006 VMSVC_XLATE_UNPIN\
    	0x62500000 VMSVC_RMLMB_KERNEL\
    	0x62500002 VMSVC_ADDLMB_KERNEL\
    	0x62500004 VMSVC_VM_ADDMEM\
    	0x62500006 VMSVC_VM_FREEMEM\
    	0x62600000 VMSVC_VM_GFREE\
    	0x62600002 VMSVC_VM_GALLOC_ATT\
    	0x62700000 VMSVC_VMS_DELETE\
    	0x62700002 VMSVC_VM_RELEASE\
    	0x62700004 VMSVC_VM_RELEASEP\
    	0x62700006 VMSVC_VM_DISCLAIM\
    	0x62800000 VMSVC_VMS_IOWAITF\
    	0x62900000 VMSVC_VM_PROTECT\
    	0x62A00000 VMSVC_VM_WRITE\
    	0x62A00002 VMSVC_VM_XWRITEP\
    	0x62B00000 VMSVC_AS_ATT64\
    	0x62B00002 VMSVC_AS_DET64\
    	0x62B00004 VMSVC_AS_SETH64\
    	0x62C00000 VMM_MIG_DR\
    	0x62C00001 VMM_MIG_DR_ERR\
    	0x62C00002 VMM_MIG_PG_DR\
    	0x62C00003 VMM_MIG_PG_DR_ERR\
    	0x62C00004 VMM_MIG_SERV_DR\
    	0x62C00005 VMM_MIG_SERV_DR_ERR\
    	0x7FF00000 PSE\
    ]
}

package provide AIXTrack $AIXTrack::version
package require AIXUtils 
package require OSInfo 

###################################################
# Hooks data base operations
###################################################

proc AIXTrack::find_hook {hookid} {
    variable hooks
    if {[llength [array names hooks $hookid]] == 0} {
        return "UNKNOWN"
    } else {
        return $hooks($hookid)
    }
}

proc AIXTrack::find_exec {pid} {
    global osinfo_pid_map
    if {[info exists osinfo_pid_map]} {
        # Change string PID to decimal number
        set pid [expr $pid | 0]
        if {[llength [array names osinfo_pid_map $pid]] == 0} {
            return "UNKOWN"
        } else {
            return $osinfo_pid_map($pid)
        }
    } else {
        return "NO MAP"
    }
}

###################################################
# Process tracking
###################################################

proc AIXTrack::initp { sim args } {
    array set p [lindex $args 0]
    set mcm $p(mcm)
    set cpu $p(cpu)
    set thread $p(thread)

    set pid [$sim cpu $cpu thread $thread display gpr 3]
    set nameaddr [$sim cpu $cpu thread $thread display gpr 7]
    set procname [AIXUtils::get_string_at_ea $sim $nameaddr $cpu $thread]
    #$sim cpu $cpu thread $thread osinfo set pid $pid exec $procname
    OSInfo::Dispatch [list mct $mcm:$cpu:$thread pid $pid exec $procname]
    #puts "******** AIXTrack::initp: cpu $cpu thread $thread pid $pid exec $procname"
}

proc AIXTrack::execvexcommon { sim args } {
    array set p [lindex $args 0]
    set mcm $p(mcm)
    set cpu $p(cpu)
    set thread $p(thread)

    set nameaddr [$sim cpu $cpu thread $thread display gpr 3]
    set filename [AIXUtils::get_string_at_ea $sim $nameaddr $cpu $thread]
    #$sim cpu $cpu thread $thread osinfo set exec $filename
    OSInfo::Dispatch [list mct $mcm:$cpu:$thread exec $filename]
    #puts "******** AIXTrack::execvexcommon: cpu $cpu thread $thread exec $filename"
}

proc AIXTrack::dispatch { sim args } {
    array set p [lindex $args 0]
    set cpu $p(cpu)
    set thread $p(thread)

    # Set up a breakpoint for dispatch return
    set lr [$sim cpu $cpu thread $thread display spr lr]
    $sim trigger set pc $lr "AIXTrack::dispatch_return $sim"
    #puts "******** AIXTrack::dispatch: cpu $cpu thread $thread return->$lr"
}

proc AIXTrack::dispatch_return { sim args } {
    array set p [lindex $args 0]
    set pc $p(address)
    set mcm $p(mcm)
    set cpu $p(cpu)
    set thread $p(thread)

    # Dispatch returns a pointer to the os thread structure (in r3)
    set tsp [$sim cpu $cpu thread $thread display gpr 3]
    #puts "******** AIXTrack::dispatch_return: tsp $tsp"
    #puts "\tt_tid\t[AIXUtils::get_data_at_ea $sim $tsp 8 $cpu $thread]"
    #puts "\tt_vtid\t[AIXUtils::get_data_at_ea $sim [expr $tsp+8] 8 $cpu $thread]"
    #puts "\tt_kthreadseg\t[AIXUtils::get_data_at_ea $sim [expr $tsp+16] 8 $cpu $thread]"
    #puts "\tt_atomic\t[AIXUtils::get_data_at_ea $sim [expr $tsp+24] 8 $cpu $thread]"
    #puts "\tt_flags\t[AIXUtils::get_data_at_ea $sim [expr $tsp+32] 4 $cpu $thread]"
    #puts "\tt_flags2\t[AIXUtils::get_data_at_ea $sim [expr $tsp+36] 4 $cpu $thread]"
    #puts "\tt_lockcount\t[AIXUtils::get_data_at_ea $sim [expr $tsp+40] 2 $cpu $thread]"
    #puts "\tt_suspend\t[AIXUtils::get_data_at_ea $sim [expr $tsp+42] 2 $cpu $thread]"
    #puts "\tPAD\t[AIXUtils::get_data_at_ea $sim [expr $tsp+44] 4 $cpu $thread]"
    #puts "\tt_pvthreadp\t[AIXUtils::get_data_at_ea $sim [expr $tsp+48] 8 $cpu $thread]"
    #puts "\tt_procp\t[AIXUtils::get_data_at_ea $sim [expr $tsp+56] 8 $cpu $thread]"

    # The first field in thread_struct is the os TID (64 bits)
    set tid [AIXUtils::get_data_at_ea $sim $tsp 8 $cpu $thread]
    #puts "******** AIXTrack::dispatch_return: tid $tid"

    # The proc struct pointer at byte-offset 56
    #set tsp_psp_addr [format "0x%016lX" [expr $tsp+56]]
    #puts "******** AIXTrack::dispatch_return: tsp_psp_addr $tsp_psp_addr"
    set psp [AIXUtils::get_data_at_ea $sim [expr $tsp+56] 8 $cpu $thread]
    #puts "******** AIXTrack::dispatch_return: psp $psp"
    #puts "\tp_pvprocp\t[AIXUtils::get_data_at_ea $sim $psp 8 $cpu $thread]"
    #puts "\tp_pid\t[AIXUtils::get_data_at_ea $sim [expr $psp+8] 8 $cpu $thread]"

    # The PID is at offset 8 within the proc struct (64 bits)
    #set pid_addr [format "0x%016lX" [expr $psp+8]]
    #puts "******** AIXTrack::dispatch_return: pid_addr $pid_addr"
    set pid [AIXUtils::get_data_at_ea $sim [expr $psp+8] 8 $cpu $thread]
    #puts "******** AIXTrack::dispatch_return: pid $pid"

    #puts "******** AIXTrack::dispatch_return: cpu $cpu thread $thread pid $pid tid $tid"
    #$sim cpu $cpu thread $thread osinfo set pid $pid tid $tid
    OSInfo::Dispatch [list mct $mcm:$cpu:$thread pid $pid tid $tid]

    # Unregister the breakpoint
    $sim trigger clear pc $pc
}

proc AIXTrack::trchk0 {sim args} {
    set myargs [lindex $args 0]
    set pc [lindex $myargs 1]
    set cpu [lindex $myargs 5]
    set thread [lindex $myargs 7]

    set hooknum [$sim cpu $cpu thread $thread display gpr 3]
    set hooknum [expr $hooknum & 0xffff0000]
    set hookstr [format "0x%X" $hooknum]
    set hooklabel [AIXTrack::find_hook $hookstr]

    # puts "******** AIXTrack::trchk0: cpu $cpu thread $thread pc $pc hook $hookstr id $hooklabel"

    if { [string equal $hookstr "0x10600000"] ||\
         [string equal $hookstr "0x10c00000"] ||\
         [string equal $hookstr "0x13900000"] } { #thread dispatch
        set pid [$sim cpu $cpu thread $thread display gpr 4]
        set tid [$sim cpu $cpu thread $thread display gpr 5]
        set execname [AIXTrack::find_exec $pid]
        $sim cpu $cpu thread $thread osinfo set pid $pid tid $tid
        puts "******** AIXTrack::trchk0(dispatch): cpu $cpu thread $thread pid $pid tid $tid ($execname)"
    } elseif { [string equal $hookstr "0x13400000"] } { #execve
        set nameaddr [$sim cpu $cpu thread $thread display gpr 6]
        set string [AIXUtils::get_string_at_ea $sim $nameaddr $cpu $thread]
        $sim cpu $cpu thread $thread osinfo set exec $string
        puts "******** AIXTrack::trchk0(execve): cpu $cpu thread exec $string"
    } elseif { [string equal $hookstr "0x13500000"] } { #process exit
        set tid [$sim cpu $cpu thread $thread display gpr 5]
        puts "******** AIXTrack::trchk0(exit): cpu $cpu thread tid $tid"
    }
}

proc AIXTrack::trchkmod_trigger {title args} {
    puts "tracehook control being modified, may lose visibility!"
    #mysim memory set 0x034C8 4 0xffffffffffffffff
    mysim memory set 0x034C8 1 0xff
}     

###################################################
# Process tracking
###################################################

proc AIXTrack::enable { {sim mysim} } {
    # Set magic trace flags (0x34c8 is address of Trconflag)
    #$sim memory set 0x138b0 4 0x1
    #$sim memory set 0x034C8 1 0xff
    #$sim trigger set memory system w  0x0034C8 0x034C8 0 "AIXTrack::trchkmod_trigger $sim"

    set initpentry [AIXUtils::get_function_entry "initp"]
    $sim trigger set pc $initpentry "AIXTrack::initp $sim"

    set execvexcommon_entry [AIXUtils::get_function_entry "execvex_common"]
    $sim trigger set pc $execvexcommon_entry "AIXTrack::execvexcommon $sim"

    #set trchk0entry [aix_get_function_entry "trchook64"]
    #defined in AIX53D an higher
    #set trchook2entry [aix_get_function_entry "mtrchook2"]
    #set trchook5entry [aix_get_function_entry "mtrchook5"]
    #$sim trigger set pc $trchk0entry "AIXTrack::trchk0 $sim"
    #$sim trigger set pc $trchook2entry "AIXTrack::trchk0 $sim" 
    #$sim trigger set pc $trchook5entry "AIXTrack::trchk0 $sim"

    set dispatch_entry [AIXUtils::get_function_entry "dispatch"]
    $sim trigger set pc $dispatch_entry "AIXTrack::dispatch $sim"

    #puts "******** AIXTrack::enable: [$sim trigger display pc list]"
}
