#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2014
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : ERAT.tcl
##
## Purpose    : Mambo ERAT display and triggering functions
##
## CVS INFO:
##
## $Id$
##
#####################################################
#####################################################

namespace eval ERAT {
    variable version            "1.0"
    variable debug              0
    variable silent             0
}

package provide ERAT $ERAT::version

####
# Display an ERAT
#
proc ERAT::display {{erat .*} {mc 0:0} {sim mysim}} {
    if {![regexp {((\d):)?(\d)} $mc match x m c]} {
        puts "ERAT::display -- malformed mcm:cpu id '$mc'"
        return -code error
    }

    if {[catch {array set erat_info [$sim mcm $m cpu $c erat display $erat]} errs]} {
        puts "ERAT::display -- cannot issue erat display command ($errs)"
        return -code error
    }

    foreach e [array names erat_info] {
        puts "ERAT: $e"
        array unset row
        array set row $erat_info($e)
        foreach r [array names row] {
            puts "\tROW: $r"
            array unset col
            array set col $row($r)
            foreach c [array names col] {
                puts "\t\tCOL: $c\t$col($c)"
            }
        }
    }
}

####
# ERAT Trigger Add
#
proc ERAT::trigger_add {args} {
    puts "ERAT::trigger_add $args"

    array set p $args
    ERAT::display $p(erat) $p(mcm):$p(cpu) $p(sim)
}

####
# ERAT Trigger central dispatch
#
proc ERAT::trigger_dispatch {args} {
    array set p $args
    switch -exact -- $p(op) {
        add {
            eval ERAT::trigger_add $args
        }
        default {
            puts "*** ERAT: $args"
        }
    }
}

####
# ERAT trigger enable
#
proc ERAT::trigger_enable {{erat .*} {mc 0:0} {sim mysim}} {
    if {![regexp {((\d):)?(\d)} $mc match x m c]} {
        puts "ERAT::trigger_enable -- malformed mcm:cpu id '$mc'"
        return -code error
    }

    if {[catch {$sim mcm $m cpu $c trigger erat set $erat [list ERAT::trigger_dispatch]} errs]} {
        puts "ERAT::trigger_enable -- cannot issue erat trigger command ($errs)"
        return -code error
    }
}
